@@ -112,6 +112,93 @@ QString AbstractMetaType::cppSignature() const
     return m_cachedCppSignature;
 }
 
+void AbstractMetaType::decideUsagePattern()
+{
+    const TypeEntry* type = typeEntry();
+
+    if (type->isPrimitive() && (!actualIndirections()
+        || (isConstant() && isReference() && !indirections()))) {
+        setTypeUsagePattern(AbstractMetaType::PrimitivePattern);
+
+    } else if (type->isVoid()) {
+        setTypeUsagePattern(AbstractMetaType::NativePointerPattern);
+
+    } else if (type->isVarargs()) {
+        setTypeUsagePattern(AbstractMetaType::VarargsPattern);
+
+    } else if (type->isString()
+               && indirections() == 0
+               && (isConstant() == isReference()
+                   || isConstant())) {
+        setTypeUsagePattern(AbstractMetaType::StringPattern);
+
+    } else if (type->isChar()
+               && !indirections()
+               && isConstant() == isReference()) {
+        setTypeUsagePattern(AbstractMetaType::CharPattern);
+
+    } else if (type->isJObjectWrapper()
+               && !indirections()
+               && isConstant() == isReference()) {
+        setTypeUsagePattern(AbstractMetaType::JObjectWrapperPattern);
+
+    } else if (type->isVariant()
+               && !indirections()
+               && isConstant() == isReference()) {
+        setTypeUsagePattern(AbstractMetaType::VariantPattern);
+
+    } else if (type->isEnum() && !actualIndirections()) {
+        setTypeUsagePattern(AbstractMetaType::EnumPattern);
+
+    } else if (type->isObject()
+               && indirections() == 0
+               && isReference()) {
+        if (((ComplexTypeEntry*) type)->isQObject())
+            setTypeUsagePattern(AbstractMetaType::QObjectPattern);
+        else
+            setTypeUsagePattern(AbstractMetaType::ObjectPattern);
+
+    } else if (type->isObject()
+               && indirections() == 1) {
+        if (((ComplexTypeEntry*) type)->isQObject())
+            setTypeUsagePattern(AbstractMetaType::QObjectPattern);
+        else
+            setTypeUsagePattern(AbstractMetaType::ObjectPattern);
+
+        // const-references to pointers can be passed as pointers
+        if (isReference() && isConstant()) {
+            setReference(false);
+            setConstant(false);
+        }
+
+    } else if (type->isContainer() && !indirections()) {
+        setTypeUsagePattern(AbstractMetaType::ContainerPattern);
+
+    } else if (type->isTemplateArgument()) {
+
+    } else if (type->isFlags()
+               && !indirections()
+               && (isConstant() == isReference())) {
+        setTypeUsagePattern(AbstractMetaType::FlagsPattern);
+
+    } else if (type->isArray()) {
+        setTypeUsagePattern(AbstractMetaType::ArrayPattern);
+
+    } else if (type->isThread()) {
+        Q_ASSERT(indirections() == 1);
+        setTypeUsagePattern(AbstractMetaType::ThreadPattern);
+    } else if (type->isValue()) {
+        if (indirections() == 1) {
+            setTypeUsagePattern(AbstractMetaType::ValuePointerPattern);
+        } else {
+            setTypeUsagePattern(AbstractMetaType::ValuePattern);
+        }
+    } else {
+        setTypeUsagePattern(AbstractMetaType::NativePointerPattern);
+        ReportHandler::debugFull(QString("native pointer pattern for '%1'")
+                                 .arg(cppSignature()));
+    }
+}
 
 /*******************************************************************************
  * AbstractMetaArgument
@@ -727,7 +814,7 @@ CodeSnipList AbstractMetaFunction::injectedCodeSnips(CodeSnip::Position position
 {
     CodeSnipList result;
     foreach (const FunctionModification mod, modifications(ownerClass())) {
-        if (mod.isCodeInjection() && TypeDatabase::instance()->supportedApiVersion(mod.version())) {
+        if (mod.isCodeInjection()) {
             QList<CodeSnip>::const_iterator it = mod.snips.constBegin();
             for (;it != mod.snips.constEnd(); ++it) {
                 if ((it->language & language) && (it->position == position || position == CodeSnip::Any))
@@ -978,6 +1065,10 @@ AbstractMetaClass::~AbstractMetaClass()
     qDeleteAll(m_fields);
     qDeleteAll(m_enums);
     qDeleteAll(m_orphanInterfaces);
+    if (hasTemplateBaseClassInstantiations()) {
+        foreach (AbstractMetaType* inst, templateBaseClassInstantiations())
+            delete inst;
+    }
 }
 
 /*******************************************************************************
@@ -1457,7 +1548,29 @@ QPropertySpec *AbstractMetaClass::propertySpecForReset(const QString &name) cons
     return 0;
 }
 
+typedef QHash<const AbstractMetaClass*, AbstractMetaTypeList> AbstractMetaClassBaseTemplateInstantiationsMap;
+Q_GLOBAL_STATIC(AbstractMetaClassBaseTemplateInstantiationsMap, metaClassBaseTemplateInstantiations);
 
+bool AbstractMetaClass::hasTemplateBaseClassInstantiations() const
+{
+    if (!templateBaseClass())
+        return false;
+    return metaClassBaseTemplateInstantiations()->contains(this);
+}
+
+AbstractMetaTypeList AbstractMetaClass::templateBaseClassInstantiations() const
+{
+    if (!templateBaseClass())
+        return AbstractMetaTypeList();
+    return metaClassBaseTemplateInstantiations()->value(this);
+}
+
+void AbstractMetaClass::setTemplateBaseClassInstantiations(AbstractMetaTypeList& instantiations)
+{
+    if (!templateBaseClass())
+        return;
+    metaClassBaseTemplateInstantiations()->insert(this, instantiations);
+}
 
 static bool functions_contains(const AbstractMetaFunctionList &l, const AbstractMetaFunction *func)
 {
