@@ -40,6 +40,7 @@ Handler::Handler(TypeDatabase* database, bool generate)
     m_current = 0;
     m_currentDroppedEntry = 0;
     m_currentDroppedEntryDepth = 0;
+    m_ignoreDepth = 0;
 
     tagNames["rejection"] = StackElement::Rejection;
     tagNames["primitive-type"] = StackElement::PrimitiveTypeEntry;
@@ -125,6 +126,11 @@ void Handler::fetchAttributeValues(const QString &name, const QXmlAttributes &at
 
 bool Handler::endElement(const QString &, const QString &localName, const QString &)
 {
+    if (m_ignoreDepth) {
+        --m_ignoreDepth;
+        return true;
+    }
+
     if (m_currentDroppedEntry) {
         if (m_currentDroppedEntryDepth == 1) {
             m_current = m_currentDroppedEntry->parent;
@@ -222,7 +228,7 @@ bool Handler::endElement(const QString &, const QString &localName, const QStrin
 
 bool Handler::characters(const QString &ch)
 {
-    if (m_currentDroppedEntry)
+    if (m_currentDroppedEntry || m_ignoreDepth)
         return true;
 
     if (m_current->type == StackElement::Template) {
@@ -380,6 +386,20 @@ static QString getNamePrefix(StackElement* element)
 bool Handler::startElement(const QString &, const QString &n,
                            const QString &, const QXmlAttributes &atts)
 {
+    if (m_ignoreDepth) {
+        ++m_ignoreDepth;
+        return true;
+    }
+
+    if (!m_defaultPackage.isEmpty() && atts.index("since") != -1) {
+        TypeDatabase* td = TypeDatabase::instance();
+        if (!td->checkApiVersion(m_defaultPackage, atts.value("since").toAscii())) {
+            ++m_ignoreDepth;
+            return true;
+        }
+    }
+
+
     QString tagName = n.toLower();
     if (tagName == "import-file")
         return importFileElement(atts);
@@ -431,8 +451,10 @@ bool Handler::startElement(const QString &, const QString &n,
             attributes["extensible"] = "no";
             attributes["identified-by-value"] = QString();
             break;
-        case StackElement::ObjectTypeEntry:
         case StackElement::ValueTypeEntry:
+            attributes["default-constructor"] = QString();
+            // fall throooough
+        case StackElement::ObjectTypeEntry:
             attributes["force-abstract"] = QString("no");
             attributes["deprecated"] = QString("no");
             attributes["hash-function"] = QString("");
@@ -546,6 +568,7 @@ bool Handler::startElement(const QString &, const QString &n,
             type->setCodeGeneration(m_generate);
             type->setTargetLangName(targetLangName);
             type->setTargetLangApiName(targetLangApiName);
+            type->setTargetLangPackage(m_defaultPackage);
             type->setDefaultConstructor(defaultConstructor);
 
             bool preferred;
@@ -645,21 +668,26 @@ bool Handler::startElement(const QString &, const QString &n,
             itype->setOrigin(otype);
             element->entry = otype;
         }
+        // fall through
+        case StackElement::ValueTypeEntry: {
+            if (!element->entry) {
+                ValueTypeEntry* typeEntry = new ValueTypeEntry(name, since);
+                QString defaultConstructor = attributes["default-constructor"];
+                if (!defaultConstructor.isEmpty())
+                    typeEntry->setDefaultConstructor(defaultConstructor);
+                element->entry = typeEntry;
+            }
+
         // fall through
         case StackElement::NamespaceTypeEntry:
             if (!element->entry)
                 element->entry = new NamespaceTypeEntry(name, since);
 
-            // fall through
+        // fall through
         case StackElement::ObjectTypeEntry:
             if (!element->entry)
                 element->entry = new ObjectTypeEntry(name, since);
 
-            // fall through
-        case StackElement::ValueTypeEntry: {
-            if (!element->entry)
-                element->entry = new ValueTypeEntry(name, since);
-
             element->entry->setStream(attributes["stream"] == QString("yes"));
 
             ComplexTypeEntry *ctype = static_cast<ComplexTypeEntry *>(element->entry);
@@ -980,6 +1008,7 @@ bool Handler::startElement(const QString &, const QString &n,
                 TypeSystemTypeEntry* moduleEntry = reinterpret_cast<TypeSystemTypeEntry*>(
                         m_database->findType(m_defaultPackage));
                 element->entry = moduleEntry ? moduleEntry : new TypeSystemTypeEntry(m_defaultPackage, since);
+                element->entry->setCodeGeneration(m_generate);
             }
 
             if ((m_generate == TypeEntry::GenerateForSubclass ||
@@ -1008,10 +1037,6 @@ bool Handler::startElement(const QString &, const QString &n,
                 return false;
             }
             QString name = attributes["name"];
-
-            if (!name.isEmpty() && m_database->supportedApiVersion(since))
-                m_currentEnum->addEnumValueRejection(name);
-
         } break;
         case StackElement::ReplaceType: {
             if (topElement.type != StackElement::ModifyArgument) {
@@ -1701,15 +1726,23 @@ PrimitiveTypeEntry* PrimitiveTypeEntry::basicAliasedTypeEntry() const
         return m_aliasedTypeEntry;
 }
 
+typedef QHash<const PrimitiveTypeEntry*, QString> PrimitiveTypeEntryTargetLangPackageMap;
+Q_GLOBAL_STATIC(PrimitiveTypeEntryTargetLangPackageMap, primitiveTypeEntryTargetLangPackages);
+
+void PrimitiveTypeEntry::setTargetLangPackage(const QString& package)
+{
+    primitiveTypeEntryTargetLangPackages()->insert(this, package);
+}
+QString PrimitiveTypeEntry::targetLangPackage() const
+{
+    if (!primitiveTypeEntryTargetLangPackages()->contains(this))
+        return this->::TypeEntry::targetLangPackage();
+    return primitiveTypeEntryTargetLangPackages()->value(this);
+}
+
 CodeSnipList TypeEntry::codeSnips() const
 {
-    CodeSnipList lst;
-    TypeDatabase *td = TypeDatabase::instance();
-    foreach(CodeSnip cs, m_codeSnips) {
-        if (td->supportedApiVersion(cs.version))
-            lst.append(cs);
-    }
-    return lst;
+    return m_codeSnips;
 }
 
 QString Modification::accessModifierString() const
@@ -1724,13 +1757,11 @@ QString Modification::accessModifierString() const
 FunctionModificationList ComplexTypeEntry::functionModifications(const QString &signature) const
 {
     FunctionModificationList lst;
-    TypeDatabase *td = TypeDatabase::instance();
     for (int i = 0; i < m_functionMods.count(); ++i) {
         const FunctionModification &mod = m_functionMods.at(i);
-        if ((mod.signature == signature) && (td->supportedApiVersion(mod.version())))
+        if (mod.signature == signature)
             lst << mod;
     }
-
     return lst;
 }
 
@@ -1745,6 +1776,26 @@ FieldModification ComplexTypeEntry::fieldModification(const QString &name) const
     return mod;
 }
 
+// The things we do not to break the ABI...
+typedef QHash<const ComplexTypeEntry*, QString> ComplexTypeEntryDefaultConstructorMap;
+Q_GLOBAL_STATIC(ComplexTypeEntryDefaultConstructorMap, complexTypeEntryDefaultConstructors);
+
+void ComplexTypeEntry::setDefaultConstructor(const QString& defaultConstructor)
+{
+    if (!defaultConstructor.isEmpty())
+        complexTypeEntryDefaultConstructors()->insert(this, defaultConstructor);
+}
+QString ComplexTypeEntry::defaultConstructor() const
+{
+    if (!complexTypeEntryDefaultConstructors()->contains(this))
+        return QString();
+    return complexTypeEntryDefaultConstructors()->value(this);
+}
+bool ComplexTypeEntry::hasDefaultConstructor() const
+{
+    return complexTypeEntryDefaultConstructors()->contains(this);
+}
+
 QString ContainerTypeEntry::targetLangPackage() const
 {
     return QString();
@@ -2095,13 +2146,20 @@ static bool strLess(const char* a, const char* b)
 
 bool TypeEntry::isCppPrimitive() const
 {
+    if (!isPrimitive())
+        return false;
     if (m_name.contains(' ') || m_type == VoidType)
         return true;
     // Keep this sorted!!
-    static const char* cppTypes[] = { "bool", "char", "double", "float", "int", "long", "short", "wchar_t"};
+    static const char* cppTypes[] = { "bool", "char", "double", "float", "int", "long", "long long", "short",
+                                      "unsigned char", "unsigned double", "unsigned float", "unsigned int",
+                                      "unsigned long", "unsigned long long", "unsigned short", "wchar_t" };
     const int N = sizeof(cppTypes)/sizeof(char*);
 
-    const char** res = qBinaryFind(&cppTypes[0], &cppTypes[N], m_name.toAscii().constData(), strLess);
+    PrimitiveTypeEntry* aliasedType = ((PrimitiveTypeEntry*)this)->basicAliasedTypeEntry();
+    QString typeName = aliasedType ? aliasedType->name() : m_name;
+
+    const char** res = qBinaryFind(&cppTypes[0], &cppTypes[N], typeName.toAscii().constData(), strLess);
     return res != &cppTypes[N];
 }
 
