@@ -1142,6 +1142,8 @@ public:
         m_preferredTargetLangType = b;
     }
 
+    void setTargetLangPackage(const QString& package);
+    QString targetLangPackage() const;
 private:
     QString m_targetLangName;
     QString m_targetLangApiName;
@@ -1439,6 +1441,7 @@ public:
         centry->setCodeSnips(codeSnips());
         centry->setTargetLangPackage(targetLangPackage());
         centry->setBaseContainerType(baseContainerType());
+        centry->setDefaultConstructor(defaultConstructor());
 
         return centry;
     }
@@ -1629,6 +1632,9 @@ public:
         return m_baseContainerType;
     }
 
+    QString defaultConstructor() const;
+    void setDefaultConstructor(const QString& defaultConstructor);
+    bool hasDefaultConstructor() const;
 
 private:
     AddedFunctionList m_addedFunctions;
